@@ -355,14 +355,18 @@ def _c14n(t, exclusive, with_comments, inclusive_prefix_list=None, schema=None):
     :param inclusive_prefix_list: List of namespaces to include (?)
     :returns: XML as string (utf8)
     """
-    xml_str = etree.tostring(t)
-    doc = parse_xml(xml_str, remove_whitespace=config.c14n_strip_ws, remove_comments=not with_comments, schema=schema)
+    doc = t
+    if root_elt(doc).getparent() is not None:
+        xml_str = etree.tostring(doc)
+        doc = parse_xml(xml_str, remove_whitespace=config.c14n_strip_ws, remove_comments=not with_comments, schema=schema)
+        del xml_str
+
     buf = etree.tostring(doc,
                          method='c14n',
                          exclusive=exclusive,
                          with_comments=with_comments,
                          inclusive_ns_prefixes=inclusive_prefix_list)
-    u = unescape_xml_entities(buf.decode("utf8", 'replace')).encode("utf8").strip()
+    u = unescape_xml_entities(buf.decode("utf8", 'strict')).encode("utf8").strip()
     if u[0] != '<':
         raise XMLSigException("C14N buffer doesn't start with '<'")
     if u[-1] != '>':
@@ -511,7 +515,7 @@ def add_enveloped_signature(t,
         root_elt(t).insert(pos, tmpl)
 
 
-def sign(t, key_spec, cert_spec=None, reference_uri='', sig_path=".//{%s}Signature" % NS['ds']):
+def sign(t, key_spec, cert_spec=None, reference_uri='', insert_index=0, sig_path=".//{%s}Signature" % NS['ds']):
     """
     Sign an XML document. This means to 'complete' all Signature elements in the XML.
 
@@ -519,6 +523,8 @@ def sign(t, key_spec, cert_spec=None, reference_uri='', sig_path=".//{%s}Signatu
     :param key_spec: private key reference, see _load_keyspec() for syntax.
     :param cert_spec: None or public key reference (to add cert to document), see _load_keyspec() for syntax.
     :param reference_uri: Envelope signature reference URI
+    :param insert_index: Insertion point for the Signature element,
+                         Signature is inserted at beginning by default
     :returns: XML as lxml.etree (for convenience, 't' is modified in-place)
     """
     do_padding = False  # only in the case of our fallback keytype do we need to do pkcs1 padding here
@@ -542,7 +548,7 @@ def sign(t, key_spec, cert_spec=None, reference_uri='', sig_path=".//{%s}Signatu
         logging.debug("Using %s bit key" % (private['keysize']))
 
     if t.find(sig_path) is None:
-        add_enveloped_signature(t, reference_uri=reference_uri)
+        add_enveloped_signature(t, reference_uri=reference_uri, pos=insert_index)
 
     if config.debug_write_to_files:
         with open("/tmp/sig-ref.xml", "w") as fd:
